/*
 *      Copyright (C) 2010 Four J's Development Tools Europe, Ltd.
 *      http://www.fourjs.com
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#ifndef _CONFIG_WIDGET_H_
#define _CONFIG_WIDGET_H_

#include <QWidget>
#include <QAbstractListModel>
#include <QTreeWidget>
#include <QStackedWidget>
#include <QDialogButtonBox>
#include <QSignalMapper>
#include <QStack>
#include <QSet>

extern "C" {
#include "putty.h"
}

class QTreeWidgetItem;
class PuttyGridLayout;
class QLayout;
class QLabel;
class ConfigWidget:public QWidget
{
    Q_OBJECT
    public:
    static bool isFixedFont(const QFont& font);
    static bool isULDFont(const QFont& font);
    static bool isValidFont(const QFont& font);
    static QFont findValidFont(const QString& text);

    ConfigWidget(bool buttonBox=true);
    ~ConfigWidget();
    virtual void fill(Config* cfg,QDialog* parent=0,int midSession=0,int protCfgInfo=0);
    QString toShortCutString(const char* label,char shortcut) const;

    void setCheckBox(union control* ctrl,int checked);
    int checkBox(union control* ctrl) const;
    void setRadioButton(union control* ctrl,int which);
    int radioButton(union control* ctrl) const;
    void setEditBox(union control* ctrl,const char* text);
    void editBox(union control* ctrl,char* buffer,int length) const;
    void setFileBox(union control* ctrl,const char* text);
    void fileBox(union control* ctrl,char* buffer,int length) const;
    void listBoxClear(union control* ctrl);
    void listBoxDel(union control* ctrl,int index);
    void listBoxAdd(union control* ctrl,const char* text);
    void listBoxAddWithId(union control* ctrl,const char* text,int id);
    int listBoxGetId(union control* ctrl,int index);
    int listBoxIndex(union control* ctrl);
    void listBoxSelect(union control* ctrl,int index);
    void setLabel(union control* ctrl,const char* text);
    void refresh(union control* ctrl);
    char* privData(union control* ctrl) const;
    char* setPrivData(union control* ctrl,size_t size);
    void selectColor(union control* ctrl,int r,int g,int b);
    void selectedColor(union control* ctrl,int* r,int* g,int* b);
    void errorMessage(const char* text);
    virtual void accept() const;
    virtual void reject() const;

    protected:
    void createTreeItemAndWidgetForString(const QString& path);
    void fillPage(struct controlset* cs);
    void fillButtonBox(QDialog* parent,struct controlset* cs);
    void addRelation(QWidget* w,union control* ctrl);
    QLayout* buddyLayout(QLabel* l,QObject* o,int widgetWidth=100) const;
    void insertControl(union control* ctrl,QStack<PuttyGridLayout*>& layoutStack,const QString& excludeKey);
    void callControlHandler(union control* ctrl,int eventType);
    void refreshControl(union control* ctrl);
    void refreshControl(union control* ctrl,const QString& excludeKey);
    void valueChanged(QWidget* sender);
    void startAction(QWidget* sender);
    void selectionChanged(QWidget* sender);
    void selectDialog(PuttyGridLayout* layout,union control* ctrl,QSignalMapper& sm,const QString& buttonText);
    void setBuddy(QLabel* l,QWidget* w);

    protected slots:
    void changePage(QTreeWidgetItem* item,QTreeWidgetItem* last);
    void buttonToggle(bool checked);
    void buttonClicked();
    void buttonClicked(QAbstractButton* button);
    void doubleClicked();
    void textChanged(const QString&);
    void selectionChange();
    void selectFile(QWidget*);
    void selectFont(QWidget*);
    
    protected:
    Config* _cfg;
    QMap<QTreeWidgetItem*,QWidget*> _itemToPage;
    QMap<QWidget*,QTreeWidgetItem*> _pageToItem;
    QHash<QString,QTreeWidgetItem*> _stringToItem;
    QMultiHash<QWidget*,union control*> _widgetToControl;
    QMultiHash<union control*,QWidget*> _controlToWidget;
    QMap<union control*,char*> _privData;
    QMap<QWidget*,QLabel*> _reversBuddy;

    QTreeWidget _category;
    QStackedWidget _pages;
    QDialogButtonBox _buttonBox;
    QSignalMapper _fileSm;
    QSignalMapper _fontSm;
    QColor _selectedColor;

    QSet<QString> _excludes;
    QDialog* _parent;
};

#endif
