/*
 *      Copyright (C) 2009 Four J's Development Tools Europe, Ltd.
 *      http://www.fourjs.com
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include "puttyGridLayout.h"
#include <QWidget>
#include <QDebug>
extern "C" {
#include "putty.h"
#include "dialog.h"
}

PuttyGridLayout::PuttyGridLayout(int colCount,int* colStretch):QGridLayout(),_colCount(colCount),_colNo(0),_rowNo(0)
{
    setContentsMargins(2,2,2,2);
    if(colStretch)
    {
        for(int i=0;i<colCount;++i)
        {
            setColumnStretch(i,colStretch[i]);
        }
    }
}

void 
PuttyGridLayout::add(QObject* x,int colNo)
{
    if(qobject_cast<QLayout*>(x))
    {
        addLayout((QLayout*)x,colNo);
    }
    else if(qobject_cast<QWidget*>(x))
    {
        addWidget((QWidget*)x,colNo);
    }
    else
    {
        qDebug() << "adding" << x << "not supported!";
    }
}

void 
PuttyGridLayout::addLayout(QLayout* l,int colNo)
{
    int validColNo=validateColNo(colNo);
    updateRowNo(validColNo);
    QLayoutItem* li=itemAtPosition(_rowNo,validColNo);
    if(li && li->layout())
    {
        PuttyGridLayout* theLi=(PuttyGridLayout*)li->layout();
        ((QGridLayout*)li->layout())->addLayout(l,theLi->rowCount(),0,1,COLUMN_SPAN(colNo));

    }
    else
    {
        QGridLayout* g=new PuttyGridLayout(1);
        QGridLayout::addLayout(g,_rowNo,validColNo,1,COLUMN_SPAN(colNo));
        g->addLayout(l,0,0);
    }
}

void 
PuttyGridLayout::addWidget(QWidget* w,int colNo)
{
    int validColNo=validateColNo(colNo);
    updateRowNo(validColNo);
    QLayoutItem* li=itemAtPosition(_rowNo,validColNo);
    if(li && li->layout())
    {
        PuttyGridLayout* theLi=(PuttyGridLayout*)li->layout();
        ((QGridLayout*)li->layout())->addWidget(w,theLi->rowCount(),0,1,COLUMN_SPAN(colNo));
    }
    else
    {
        QGridLayout* g=new PuttyGridLayout(1);
        QGridLayout::addLayout(g,_rowNo,validColNo,1,COLUMN_SPAN(colNo));
        g->addWidget(w,0,0);
    }
}

int
PuttyGridLayout::validateColNo(int colNo)
{
    if(colNo>0xffff)
    {
        return COLUMN_START(colNo);
    }
    if(colNo>=_colCount)
    {
        colNo%=_colCount;
    }
    return colNo;
}

void
PuttyGridLayout::updateRowNo(int colNo)
{
    if(_colNo>colNo)
    {
        ++_rowNo;
    }
    _colNo=colNo;
}
