/*
 *      Copyright (C) 2009 Four J's Development Tools Europe, Ltd.
 *      http://www.fourjs.com
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#ifndef _Q_PUTTY_H_
#define _Q_PUTTY_H_

#include <QWidget>
#include <QScrollBar>
#include <QPalette>
#include <QGridLayout>
#include <QStringList>
#include "structs.h"

class AbstractTerminalWidget;
class QMenu;
class QTemporaryFile;
class QPutty: public QWidget
{
    Q_OBJECT
    public:
    QPutty(QWidget* parent=0);
    virtual ~QPutty();
    int run(const QStringList& args);
    QString defaultTitle(const QString& hostname) const;
    void setTitle(const QString&);
    void updateScrollBar(int length,int value,int pageStep);
    void updateCursorVisibility(int x,int y);
    void setCursor(int col,int line,const QString& text,unsigned long attr);
    void setSelectedText(const QString& text,bool mustDeselect);
    virtual void insertText(int col,int line,const QString& text,unsigned long attr);
    virtual void close(int exitCode);
    virtual void scroll(int lines);
#ifdef Q_WS_WIN
    bool winEvent(MSG* msg,long* result);
#else
    int registerFd(int fd,int rwx);
    void releaseFd(int id);
#endif
    void timerChangeNotify(long ticks,long nextNow);
    virtual void eventLogUpdate(int eventNo);
    virtual bool isAlwaysAcceptHostKey() const;

    public slots:
    virtual void requestPaste();
    virtual void showEventLog();
    virtual void about();
    virtual void reconfigure();
    virtual void newSession();
    virtual void dupSession();
    virtual void savedSession();
    virtual void setTitle();

    signals:
    void finished(int exitCode);

    protected:
    virtual void closeEvent(QCloseEvent* event);
    static char** toArgv(const QStringList& args);
    void resizeEvent(QResizeEvent*);
    void changeEvent(QEvent*);
    virtual void contextMenuEvent(QContextMenuEvent* event);
    virtual AbstractTerminalWidget* createTerminalWidget();
    virtual void setupPutty();
    virtual void setupDefaults();
    virtual int parseCommandLine(const QStringList& args);
    virtual int showConfigDialog();
    virtual void setupTerminalControl();
    virtual void setupScrollBar();
    virtual void setupLogging();
    virtual void setupFonts();
    virtual void setupPalette();
    virtual void setupOverwriteDefaults();
    virtual void setupWidget();
    virtual void startDetachedProcess(const QString& cmd);
    virtual QMenu* savedSessionContextMenu() const;
    virtual bool isScalingMode();

    private:
#ifdef Q_WS_WIN
    void setupDefaultFontName(FontSpec& fs);
#endif

    protected:
    AbstractTerminalWidget* _terminalWidget;
    QScrollBar _vBar;
    QPalette _palette;
    QGridLayout _layout;
    struct gui_data _inst;
    bool _running;
    int _shownLogEvents;
    int _eventLogCount;
    QSize _oldSize;
    QTemporaryFile* _ppk;
};

#endif
