/*
 *      Copyright (C) 2009 Four J's Development Tools Europe, Ltd.
 *      http://www.fourjs.com
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include "terminalCursor.h"
#include <QWidget>
#include <QPainter>
#include "terminalWidget.h"

namespace PTerminal
{
Cursor::Cursor(Widget* parent):QObject(parent),_shape(Empty),_shapeWidth(2),_attr(0)
{
}

Cursor::~Cursor()
{
}

int
Cursor::col() const
{
    return widget()->_inst->term->dispcursx;
}

int
Cursor::line() const
{
    return widget()->_inst->term->dispcursy;
}

void
Cursor::setText(const QString& text,unsigned long attr)
{
    _text=text;
    _attr=attr;
    QRect r(col()*widget()->_inst->font_width,line()*widget()->_inst->font_height,widget()->_inst->font_width,widget()->_inst->font_height);
    switch(_shape)
    {
        case Block:
            _rect=r;
            break;
        case Underline:
            _rect=r.adjusted(0,r.height()-_shapeWidth,0,0);
            break;
        case Vertical:
            _rect=r.adjusted(0,0,-r.width()+_shapeWidth,0);
            break;
        default:
            _rect=r.adjusted(0,0,-r.width(),-r.height());
    }
    widget()->update(widget()->_transform.mapRect(_rect));
}

void
Cursor::setShape(int shape)
{
    if(shape==_shape) return;
    switch(shape)
    {
        case 0:
            _shape=Block;
            break;
        case 1:
            _shape=Underline;
            break;
        case 2:
            _shape=Vertical;
            break;
        default:
            _shape=Empty;
    }
}

void
Cursor::draw(QPainter& p)
{
    if(col()==-1 || line()==-1) return;
    p.setFont(((QWidget*)parent())->font());
    if(_attr&TATTR_PASCURS)
    {
        p.setPen(widget()->fgColor(_attr,true));
        p.drawRect(_rect.adjusted(0,0,-1,-1));
        p.setPen(widget()->fgColor(_attr,true));
        p.drawText(_rect,_text);
    }
    else if(_attr&TATTR_ACTCURS && widget()->_inst->term->cblinker)
    {
        p.fillRect(_rect,widget()->bgColor(_attr,true));
        p.setPen(widget()->fgColor(_attr,true));
        p.drawText(_rect,_text);
    }
}

Widget*
Cursor::widget() const
{
    return (Widget*)parent();
}
}
