/*
 *      Copyright (C) 2010 Four J's Development Tools Europe, Ltd.
 *      http://www.fourjs.com
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation, 
 *  Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 */
#include "terminalText.h"
#include <QWidget>
#include <QPainter>
#include <QString>
#include <QColor>
#include <QFont>
#include <QMetaEnum>
#include "terminalWidget.h"

namespace PTerminal
{
Text::Text(Widget* p):QObject(p),_fontSize(0,0)
{
}

void
Text::init()
{
    _fontSize.setX((qreal)widget()->_inst->font_width);
    _fontSize.setY((qreal)widget()->_inst->font_height);

    _ldcMap.clear();

    QPolygon polygon;

    // -
    polygon << QPoint(0,_fontSize.y()/2) << QPoint(_fontSize.x(),_fontSize.y()/2);
    _ldcMap.insert(lh,polygon);

    // |
    polygon.clear();
    polygon << QPoint(_fontSize.x()/2,0) << QPoint(_fontSize.x()/2,_fontSize.y());
    _ldcMap.insert(lv,polygon);

    //  _
    // |
    polygon.clear();
    polygon << QPoint(_fontSize.x()/2,_fontSize.y()/2) << QPoint(_fontSize.x()/2,_fontSize.y());
    polygon << QPoint(_fontSize.x()/2,_fontSize.y()/2) << QPoint(_fontSize.x(),_fontSize.y()/2);
    _ldcMap.insert(ldlr,polygon);

    //  _
    //  | 
    polygon.clear();
    polygon << QPoint(_fontSize.x()/2,_fontSize.y()/2) << QPoint(_fontSize.x()/2,_fontSize.y());
    polygon << QPoint(0,_fontSize.y()/2) << QPoint(_fontSize.x()/2,_fontSize.y()/2);            
    _ldcMap.insert(ldll,polygon);

    // |_
    polygon.clear();
    polygon << QPoint(_fontSize.x()/2,0) << QPoint(_fontSize.x()/2,_fontSize.y()/2);
    polygon << QPoint(_fontSize.x()/2,_fontSize.y()/2) << QPoint(_fontSize.x(),_fontSize.y()/2);
    _ldcMap.insert(lulr,polygon);

    // _|
    polygon.clear();
    polygon << QPoint(_fontSize.x()/2,0) << QPoint(_fontSize.x()/2,_fontSize.y()/2);
    polygon << QPoint(0,_fontSize.y()/2) << QPoint(_fontSize.x()/2,_fontSize.y()/2);
    _ldcMap.insert(lull,polygon);

    // |-
    polygon.clear();
    polygon << QPoint(_fontSize.x()/2,0) << QPoint(_fontSize.x()/2,_fontSize.y());
    polygon << QPoint(_fontSize.x()/2,_fontSize.y()/2) << QPoint(_fontSize.x(),_fontSize.y()/2);
    _ldcMap.insert(lvlr,polygon);

    // -|
    polygon.clear();
    polygon << QPoint(_fontSize.x()/2,0) << QPoint(_fontSize.x()/2,_fontSize.y());
    polygon << QPoint(0,_fontSize.y()/2) << QPoint(_fontSize.x()/2,_fontSize.y()/2);
    _ldcMap.insert(lvll,polygon);

    // _
    // |
    polygon.clear();
    polygon << QPoint(0,_fontSize.y()/2) << QPoint(_fontSize.x(),_fontSize.y()/2);
    polygon << QPoint(_fontSize.x()/2,_fontSize.y()/2) << QPoint(_fontSize.x()/2,_fontSize.y());
    _ldcMap.insert(lhld,polygon);

    // |
    // - 
    polygon.clear();
    polygon << QPoint(0,_fontSize.y()/2) << QPoint(_fontSize.x(),_fontSize.y()/2);
    polygon << QPoint(_fontSize.x()/2,0) << QPoint(_fontSize.x()/2,_fontSize.y()/2);
    _ldcMap.insert(lhlu,polygon);

    // +
    polygon.clear();
    polygon << QPoint(0,_fontSize.y()/2) << QPoint(_fontSize.x(),_fontSize.y()/2);
    polygon << QPoint(_fontSize.x()/2,0) << QPoint(_fontSize.x()/2,_fontSize.y());
    _ldcMap.insert(lhlv,polygon);
}

bool
Text::isLdc(const QChar& c) const
{
    return (c.unicode()>=begin && c.unicode()<=end);
}

void
Text::draw(QPainter& p,const QRect& r,const QString& text,unsigned long attr)
{
    p.fillRect(r,widget()->bgColor(attr));
    p.setFont(widget()->font(attr));
    if(isLdc(text.at(0)))
    {
        p.setPen(QPen(widget()->fgColor(attr),(attr&ATTR_BOLD)?2:1));
        LDC key;
        QMetaEnum me=metaObject()->enumerator(metaObject()->indexOfEnumerator("LDC"));
        for(int i=2;i<me.keyCount();++i)
        {
            if(text.at(0).unicode()<me.value(i))
            {
                key=(LDC)me.value(i-1);
                break;
            }
        }
        if(text.at(0).unicode()==end)
        {
            key=lhlv;
        }
        const QPolygon& polygon=_ldcMap.value(key);
        for(int j=0;j<text.size();++j)
        {
            p.drawLines(polygon.translated(r.topLeft().x()+j*_fontSize.x(),r.topLeft().y()));
        }
    }
    else
    {
        p.setPen(widget()->fgColor(attr));
        QFontMetrics fm(widget()->font(attr));
        QRect br=fm.boundingRect(text);
        p.drawText(r.x(),r.y()-br.y(),text);
    }
}

Widget*
Text::widget() const
{
    return (Widget*)parent();
}

}
